var mongoose = require('mongoose')
var crypto = require('crypto')
var jwt = require('jsonwebtoken')

var userSchema = new mongoose.Schema({
  email: {
    type: String,
    unique: true,
    required: true
  },
  emailVerified: {
      type: Boolean,
      required: false
  },
  firstName: {
    type: String,
    required: true
  },
  lastName: {
    type: String,
    required: true
  },
  middleName: {
    type: String,
    required: false
  },
  phone: {
    type: String,
    required: true
  },
  phoneVerified: {
      type: Boolean,
      required: false
  },
  hash: String,
  salt: String
})

userSchema.methods.setPassword = function(password){
  try {
    this.salt = crypto.randomBytes(16).toString('hex')
    this.hash = crypto.pbkdf2Sync(password, this.salt, 1000, 512, 'sha512').toString('hex')
  }
  catch(e) {
    console.log(e.message)
  }
}

userSchema.methods.validPassword = function(password) {
  try {
    var hash = crypto.pbkdf2Sync(password, this.salt, 1000, 512, 'sha512').toString('hex')
    return this.hash === hash
  }
  catch(e) {
    console.log(e.message)
  }
}

userSchema.methods.generateJwt = function() {
  try {
    var expiry = new Date()
    expiry.setDate(expiry.getDate() + 7)

    return jwt.sign({
      _id: this._id,
      email: this.email,
      fullName: this.fullName,
      phone: this.phone,
      exp: parseInt(expiry.getTime() / 1000),
    }, process.env.SESSION_SECRET); // DO NOT KEEP YOUR SECRET IN THE CODE!
  }
  catch(e) {
    console.log(e.message)
  }
}

mongoose.model('User', userSchema)
